<?php
/**
* Scheduler class
* This file contians the scheduler application where
*  users have an interface for reserving resources,
*  viewing other reservations and modifying their own.
* @author Nick Korbel <lqqkout13@users.sourceforge.net>
* @version 12-22-03
* @package phpScheduleIt
*/
/**
* Base directory of application
*/
@define('BASE_DIR', dirname(__FILE__) . '/..');
/**
* Include ScheduleDB class
*/
require_once('db/ScheduleDB.class.php');
/**
* Include Schedule template files
*/
require_once(BASE_DIR . '/templates/schedule.template.php');

class Schedule {

	var $_date = array();
	var $machids = array();
	var $res = array();
	var $db;
	var $user;
	
	/**
	* Sets up initial variable values
	* @param none
	*/
	function Schedule() {
		$this->db = new ScheduleDB();					// Set database class
		$this->user = new User($_SESSION['sessionID']);	// Set User class
		
		$this->_date = $this->get_date_vars();		// Get all date info we need
		$this->machids = $this->db->get_mach_ids();	// Get all resource info
		$this->res = $this->db->get_all_res($this->_date['firstDayTs'], $this->_date['lastDayTs']);	// Get all reservation data

	}
	
	/**
	* Prints the actual schedule by calling all necessary class
	*  and schedule template functions
	* @param none
	*/
	function print_schedule() {
		global $conf;
		
		print_date_span($this->_date);
		print_color_key();
		
		// Break first day we are viewing into an array of date pieces
		$temp_date = getdate($this->_date['firstDayTs']);
		$hour_header = get_hour_header($this->get_time_array());	// Get the headers (same for all tables)
		
		// Repeat this for each day we need to show
		for ($dayCount = 0; $dayCount < $conf['app']['viewDays']; $dayCount++) {
		    // Timestamp for whatever day we are currently viewing
        	$this->_date['current'] = mktime(0,0,0, $temp_date['mon'], $temp_date['mday'] + $dayCount, $temp_date['year']);
			start_day_table($this->get_display_date(), $hour_header);	// Start the table for this day
			$this->print_reservations();	// Print reservations for this day
			end_day_table();				// End the table for this day
		}
	}
	
	/**
	* Print out the reservations for each resource on each day
	* @param none
	*/
	function print_reservations() {
		global $conf;
		
		if (!$this->machids)
			return;
		
		// Repeat this whole process for each resource in the database
    	for ($count = 0; $count < count($this->machids); $count++) {
			$prevTime = $conf['app']['startDay'];		// Previous time holder
        	$totCol = intval(($conf['app']['endDay'] - $conf['app']['startDay']) / $conf['app']['timeSpan']);	// Total columns holder
			
			// Store info about this current resource in local vars
			$id = $this->machids[$count]['machid'];
			$name = $this->machids[$count]['name'];
			$status = $this->machids[$count]['status'];
			
			$shown = false;		// Default resource visiblilty to not shown
			
			// If the date has not passed, resource is active and user has permission, 
			//  or the user is the admin allow reservations to be made
			if ( $this->_date['current'] >= mktime(0,0,0) && ( ($status == 'a' && $this->user->has_perm($id)) || Auth::isAdmin()))
                $shown = true;
			
			print_name_cell($this->_date['current'], $id, $name, $shown);
			
			$index = $this->_date['current'] . '|' . $id;
			if (isset($this->res[$index])) {
				for ($i = 0; $i < count($this->res[$index]); $i++) {
					$rs = $this->res[$index][$i];
					 // Print out row of reservations
					$thisStart = $rs['startTime'];
					$thisEnd = $rs['endTime'];
					$colspan = intval(($thisEnd - $thisStart) / $conf['app']['timeSpan']);
					
					move_to_starting_col($thisStart, $prevTime, $conf['app']['timeSpan']);		
					
					write_reservation($rs, $this->_date['current'], $colspan);
								
					// Set prevTime to this reservation's ending time
					$prevTime = $thisEnd;
				}
			}
			
			finish_row($prevTime);
		}
	}
	
	/**
	* Return the formatted date
	* @param int $ts time stamp for date to format
	* @return formatted date
	*/
	function get_display_date() {
		$t = getdate($this->_date['current']);
		return sprintf('%s,<br />%02d/%02d/%04d', $t['weekday'], $t['mon'], $t['mday'], $t['year']);
	}
	
	/**
	* Sets up all date variables needed in the scheduler
	* @param none
	* @return array of all needed date variables
	*/
	function get_date_vars() {
		global $conf;
		
		$dv = array();
		
		// For Back, Current, Next Week clicked links
		//    pull values into an array month,day,year
		$indate = (isset($_GET['date'])) ? explode('-',$_GET['date']) : '';
		
		// Set date values if a date has been passed in (these will always be set to a valid date)
		if ( !empty($indate) || isset($_POST['jumpForm']) ) {
			$dv['month']  = (isset($_POST['jumpMonth'])) ? date('m', mktime(12,1,1,$_POST['jumpMonth'],1)) : date('m', mktime(12,1,1,$indate[0],1));
			$dv['day']    = (isset($_POST['jumpDay'])) ? date('d', mktime(12,1,1,$dv['month'], $_POST['jumpDay'])) : date('d', mktime(12,1,1, $dv['month'], $indate[1]));
			$dv['year']   = (isset($_POST['jumpYear'])) ? date('Y', mktime(12,1,1, $dv['month'], $dv['day'], $_POST['jumpYear'])) : date('Y', mktime(12,1,1, $dv['month'], $dv['day'], $indate[2]));
		}
		else {   
			// Else set values to current date
			$d = getdate();
			$dv['month']  = $d['mon'];
			$dv['day']    = $d['mday'];
			$dv['year']   = $d['year'];
		}
		
		// Make timestamp for today's date
		$dv['todayTs'] = mktime(0,0,0, $dv['month'], $dv['day'], $dv['year']);
		
		// Set it to first day of week
		// by subtracting day of the week from $day and adding the day the week starts on
		$dayNo = date('w', $dv['todayTs']);
		
		$dv['firstDayTs'] = mktime(0,0,0, $dv['month'], ($dv['day'] - $dayNo + $conf['app']['weekStartDay']), $dv['year']);
	
		// Make timestamp for last date
		// by adding # of days to view minus the day of the week to $day
		$dv['lastDayTs'] = mktime(0,0,0, $dv['month'], ($dv['day'] + $conf['app']['viewDays'] - $dayNo - 1 + $conf['app']['weekStartDay']), $dv['year']);
		
		$dv['current'] = $dv['firstDayTs'];

		return $dv;
		
	}
	
	
	/**
	* Get associative array of available times and rowspans
	* This function computes and returns an associative array
	* containing a time value and it's rowspan value as
	* $array[time] => rowspan
	* @param none
	* @return array of time value and it's associated rowspan value
	* @global $conf
	* @see CmnFns::formatTime()
	*/
	function get_time_array() {
		global $conf;
		
		$startDay = $startingTime = $conf['app']['startDay'];
		$endDay   = $endingTime   = $conf['app']['endDay'];
		$interval = $conf['app']['timeSpan'];
		$timeHash = array();
		
		// Compute the available times
		$prevTime = $startDay;
		
		// Set up time array with sub0 = hour, sub1 = minute
		/*
		if (!strstr($startDay,'.')) {
			$timeArray[0] = $startDay;
			$timeArray[1] = 0;
		}
		else
			$timeArray = explode('.',$startDay);
		*/
		
		/********************************************
		* First and last columns will be special cases
		* since the start and end of the day may be on a half hour.
		* Because of this, we should compute first and last cols
		* individually
		********************************************/
		
		// $min_in_day = 1440;
		/*
		// If the first time is not on an hour, do a special case and get to the first
		if (($startDay % 60) != 0) {
			// Sub from 60
			$time = CmnFns::formatTime($startDay);
			$timeHash[$time] = intval(($startDay % 60) / $conf['app']['timeSpan']);
			$prevTime = $startingTime = ($startDay % 60) + $startDay;		
		}
		
		if (($endDay % 60) != 0)
			$endingTime = $endDay - ($endDay % 60);
				
		// Loop through all time slots
		while ($prevTime < $endingTime) {			
			if (($prevTime % 60) == 0) {							// Only print out hour marks
				$time = CmnFns::formatTime($prevTime);
				$colspan = intval(60 / $conf['app']['timeSpan']);
				if ($colspan < 1) $colspan = 1;
				$timeHash[$time] = $colspan;
			}
			//echo $prevTime . ' ';
			$prevTime += $conf['app']['timeSpan'];
		}
		
		// If the last time is not on an hour
		if (($endDay % 60) != 0) {
			// Sub from 60, store last value
			$time = CmnFns::formatTime($endDay);
			$timeHash[$time] = intval(($endDay % 60) / $conf['app']['timeSpan']);
			//$endingTime = $endDay + ($endDay % 60);
		
		}
		*/
		// If the first time does not start on a whole hour and the interval is less than 60
		if ( (($startDay % 60) != 0) && ($interval < 60) ) {
			$time = CmnFns::formatTime($startDay);
			$timeHash[$time] = intval(($startDay % 60) / $interval);
			$prevTime = ($startDay % 60) + $startDay;		
		}

		while ($prevTime < $endingTime) {									
			if ($interval < 60) {				
				$time = CmnFns::formatTime($prevTime);
				$timeHash[$time] = intval(60 / $interval);
				//echo $prevTime . ' ';
				$prevTime += 60;		// Always increment by 1 hour
			}
			else {
				$colspan = 1;				// Colspan is always 1
				$time = CmnFns::formatTime($prevTime);
				$timeHash[$time] = $colspan;
				$prevTime += $interval;
			}				
		}
		
		// If the last time is not on an hour, do a special case to get to the last
		/*
		// Compute first column
		if ($timeArray[1] != 0) {
			// If the minutes is .5, then print 1 col and increment prevTime by .5
			$time = CmnFns::formatTime($prevTime);
			$timeHash[$time] = 1;
			$prevTime += .5;
		}
		// Else print col with rowspan 2 (to cover both :30 min marks)
		else {
			$time = CmnFns::formatTime($prevTime);
			$timeHash[$time] = 2;
			$prevTime++;
		}
	
		// Repeat for all but last column
		while ($prevTime < $endDay-1) {
			$time = CmnFns::formatTime($prevTime);
			$timeHash[$time] = 2;
			$prevTime++;
		}
		
		// This is the same logic as the startDay operations
		if (!strstr($endDay,'.')) {
			$timeArray[0] = $endDay;
			$timeArray[1] = 0;
		}
		else
			$timeArray = explode('.',$endDay);
		
		// Print last time column depending on endDay time
		if ($timeArray[1] != 0) {
			$time = CmnFns::formatTime($prevTime);
			$timeHash[$time] = 1;
		}
		else {
			$time = CmnFns::formatTime($prevTime);
			$timeHash[$time] = 2;
		}
		*/
		return $timeHash;   
	}
	
	/**
	* Print out links to jump to new dates
	* @param none
	*/
	function print_jump_links() {
		print_jump_links($this->_date['firstDayTs']);
	}
}
?>