<?php
/**
* This file provides the output functions for 
*  an interface for reserving resources,
*  viewing other reservations and modifying their own.
* @author Nick Korbel <lqqkout13@users.sourceforge.net>
* @version 12-22-03
* @package Templates
*/
/**
* Base directory of application
*/
@define('BASE_DIR', dirname(__FILE__) . '/..');
/**
* Include CmnFns class
*/
require_once(BASE_DIR . '/lib/CmnFns.class.php');

// Get Link object
$link = CmnFns::getNewLink();

/**
* Print out week being viewed above schedule tables
* @param array $d array of date information about this schedule
*/
function print_date_span($d) {
	// Print out current week being viewed
	echo '<h3 align="center">' . CmnFns::formatDate($d['firstDayTs']) . ' - ' . CmnFns::formatDate($d['lastDayTs']) . '</h3>';
}

/**
* Print out a key to identify what the colors mean
* @param none
*/
function print_color_key() {
	global $conf;
?>
<table align="center" cellpadding="5" cellspacing="10">
  <tr style="font-size: 10px; font-weight: bold; text-align: center; vertical-align: center;">
    <td style="width: 75px; height: 38px; background-color:#<?=$conf['ui']['my_res'][0]['color']?>; border: 2px #000000 solid;">My Reservations</td>
	<td style="width: 75px; height: 38px; background-color:#<?=$conf['ui']['my_past_res'][0]['color']?>; border: 2px #000000 solid;">My Past Reservations</td>
	<td style="width: 75px; height: 38px; background-color:#<?=$conf['ui']['other_res'][0]['color']?>; border: 2px #000000 solid;">Other Reservations</td>	
	<td style="width: 75px; height: 38px; background-color:#<?=$conf['ui']['other_past_res'][0]['color']?>; border: 2px #000000 solid;">Other Past Reservations</td>
  </tr>
  <tr>
    <td></td>
	<td></td>
	<td></td>	
	<td></td>
  </tr>
</table>
<?
}


/**
* Start table for one day on schedule
* This function starts the table for each day
* on the schedule, printing out it's date
* and the time value cells
* @param string $displayDate date string to print
*/
function start_day_table($displayDate, $hour_header) {

?>
    <table width="100%" border="0" cellspacing="0" cellpadding="1">
     <tr class="tableBorder">
      <td>
       <table width="100%" border="0" cellspacing="1" cellpadding="0">
        <tr class="scheduleTimes">
         <td rowspan="2" width="15%" class="scheduleDate"><?= $displayDate ?></td>
<?
	echo $hour_header ."</tr>\n";
}

/**
* Formats and returns the time header of the table (it is the same for every one)
* @param array $th array of time values and their rowspans
* @global $conf
*/
function get_hour_header($th) {
	global $conf;
    $header = '';
	
    // Write out the available times   
    foreach ($th as $time => $cols) {
        $header .= "<td colspan=\"$cols\">$time</td>\n";
	}
    
    // Close row, start next
    $header .= "</tr>\n<tr class=\"scheduleTimes\">\n";
    
    // Compute total # of cols
    //$tmp = intval(($conf['app']['endDay'] - $conf['app']['startDay']) / 60);
	$totCol = intval(($conf['app']['endDay'] - $conf['app']['startDay']) / $conf['app']['timeSpan']);
    //die($totCol);
    // Create the fraction hour minute marks
    for ($x = 0; $x < $totCol; $x++)
	    $header .= '<td>&nbsp;</td>';
		
	return $header;

}



/**
* Close off table for each day
* This function simply prints out the HTML to close off
* the rows and tables for each day
* @param none
*/
function end_day_table() {
?>
    </table>
   </td>
  </tr>
 </table>
 <p>&nbsp;</p>
<?
}


/**
* Prints out the cell containing all the resource information
* @param int $ts timestamp for the current day
* @param string $id id of this resource
* @param string $name name of this resource
* @param boolean $shown whether this resource can be reserved
*/
function print_name_cell($ts, $id, $name, $shown) {
	global $link;
	
	// Start a new row and print out resource name
	echo "<tr class=\"cellColor\">\n"
		   . '<td class="resourceName">';
		   
	// If the user is allowed to make reservations on this resource
	// then provide a link
	// Else do not
	if ($shown)	
		$link->doLink("javascript: reserve('r','$id','$ts','');", $name, '', '', "Reserve $name on " . date('M d, Y', $ts));
	else
		echo '<span class="inact">' . $name . '</span>';
	// Close cell	
	echo "</td>\n";	
}

/**
* Prints out all the columns that exist before the first reservation of the day
* @param int $start starting time of the day
* @param int $prev time of first reservation
* @param int $timeSpan minutes between each reservation
*/
function move_to_starting_col($start, $prev, $timeSpan) {
	// Move to starting column
	for ($i = 1; $i <= (($start-$prev) / $timeSpan); $i++)
		echo '<td>&nbsp</td>';
}


/**
* Writes out the reservation cell
* @param array $rs array of reservation information
* @param int $ts the timestamp for this reservation's date
* @param int $colspan column span value
*/
function write_reservation($rs, $ts, $colspan) {
	global $conf;
	
	$is_mine = false;
	$is_past = false;
	$color_select = 'other_res';		// Default color (if anything else is true, it will be changed)
	
	if ($rs['memberid'] == $_SESSION['sessionID']) {
		$is_mine = true;
		$color_select = 'my_res';
	}
	
	if (mktime(0,0,0) > $ts) {		// If todays date is still before or on the day of this reservation
		$is_past = true;
		$color_select = ($is_mine) ? 'my_past_res' : 'other_past_res';		// Choose which color array to use		
	}		
	
	// If this is the user who made the reservation or the admin,
	//  and time has not passed, allow them to edit it
	//  else only allow view
	$mod_view = ( ($is_mine || Auth::isAdmin()) && !$is_past) ? 'm' : 'v';	// To use in javascript edit/view box
	$color = '#' . $conf['ui'][$color_select][0]['color'];
	$hover = '#' . $conf['ui'][$color_select][0]['hover'];
	// Write reserved time cell
	echo "<td colspan=\"$colspan\" style=\"background-color: $color;\" onclick=\"javascript: reserve('$mod_view','','','{$rs['resid']}');\" onmouseover=\"javascript: this.style.backgroundColor='$hover'; this.style.cursor='hand';\" onmouseout=\"javascript: this.style.backgroundColor='$color';\">&nbsp;</td>\n";
}


/**
* Prints all remaining cells for this row
* @param int $prev previous time value
*/
function finish_row($prev) {
	global $conf;
	
	// Write remaining empty columns
	for ($i = 1; $i <= (($conf['app']['endDay']-$prev) / $conf['app']['timeSpan']); $i++)
		echo '<td>&nbsp;</td>';
	
	echo '</tr>';
}

/**
* Print links to jump to new dates
* This function prints out the HTML links to allow
*  users to navigate back/forward one week.
* It also prints the form for users to jump to
*  any given week.
* @param int $_date timestamp of first day of week on schedule
*/
function print_jump_links($_date) {
	global $link;
	global $conf;
	
	$date = getdate($_date);
	$m = $date['mon'];
	$d = $date['mday'];
	$y = $date['year'];
    
    // Write out the previous, today and next links and the form to jump to a date
?>

    <table width="60%" border="0" cellspacing="0" cellpadding="5" align="center">
     <tr>
      <td align="center"><h5><? $link->doLink($_SERVER['PHP_SELF'] . '?date=' . date('m-d-Y',mktime(12,1,1,$m, $d - $conf['app']['viewDays'], $y)), '&laquo; Prev Week', '', '', 'Jump 1 week back') ?></h5></td>
      <td align="center"><h5><? $link->doLink($_SERVER['PHP_SELF'] . '?date=' . date('m-d-Y'), 'This Week', '', '', 'Jump to this week') ?></h5></td>
      <td align="center"><h5><? $link->doLink($_SERVER['PHP_SELF'] . '?date=' . date('m-d-Y',mktime(12,1,1,$m, $d + $conf['app']['viewDays'], $y)), 'Next Week &raquo;', '', '', 'Jump 1 week forward') ?></h5></td>
     </tr>
     <tr>
      <td align="center" colspan="3">
       <form name="jumpWeek" method="post" action="<?= $_SERVER['PHP_SELF']?>" onsubmit="return checkDate();">
         <input type="text" name="jumpMonth" value="mm" class="textbox" size="3" maxlength="2" onclick="this.value = '';" />
         <input type="text" name="jumpDay" value="dd" class="textbox" size="3" maxlength="2" onclick="this.value = '';" />
         <input type="text" name="jumpYear" value="yyyy" class="textbox" size="5" maxlength="4" onclick="this.value = '';" />
         <input name="jumpForm" type="submit" value="Jump To Date" class="button" />
       </form>
      </td>
     </tr>
    </table>	
	<h4 align="center"><? $link->doLink("javascript: window.open('popCalendar.php','calendar','width=260,height=250,scrollbars=no,location=no,menubar=no,toolbar=no,resizable=yes'); void(0);", 'View Monthly Calendar', '', '', 'Open up a navigational calendar')?></h4>	
<?
}
?>